/*
 Wireless Gameboy Controller Cart
 Version: 1.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 23/09/2018
 Last Modified: 26/10/2018
 
 Use your Gameboy, GBC, GBP, GBA or GBA SP as a Wireless game controller for any device which supports USB HID Joysticks.
 This would allow you to play games on your PC with your a Gameboy of your choosing.
 
 */

#include <stdio.h>
#include <gb/gb.h>
#include <gb/console.h>
#include <gb/drawing.h>

void init();
void options();
void checkInput();
WORD senddata(WORD);
WORD updatenrf(WORD);

UWORD kpInput = 0x00;
UBYTE refreshScreen = 1;
UBYTE transmitOptions = 0;
UBYTE inOptions = 0;
UBYTE arrowLocation = 1;
UWORD rfChannel = 1;
UWORD rfAddress = 1;

void main() {
	init();
	printf("\n   insideGadgets\nGB Controller v1.1\n");
	
	// If start press at boot, go to options
	if (joypad() & J_START) {
		inOptions = 1;
	}
	
	while (1) {
		if (inOptions == 1) {
			options();
		}
		else {
			checkInput();
		}
	}
}

void init() {
	DISPLAY_ON;		// Turn on the display
	//NR52_REG = 0x8F;	// Turn on the sound
	//NR51_REG = 0x11;	// Enable the sound channels
	//NR50_REG = 0x00;	// Increase the volume to its max
}

void options() {
	if (refreshScreen == 1) {
		gotoxy(0,3);
		printf("  --- Options ---\n");
		printf("  RF Channel: %d  \n", rfChannel);
		printf("  RF Address: %d  \n", rfAddress);
		printf("\nPress A to transmit\n");
		printf("Power on receiver\n");
		printf("Wait 2-3 seconds\n");
		printf("Power cycle Gameboy\n");
		
		if (transmitOptions == 1) {
			printf("\n\nTransmitting...");
		}
		
		gotoxy(0,arrowLocation+3);
		printf(">");
		
		wait_vbl_done();
		refreshScreen = 0;
	}
	
	// RF channel/address
	if (transmitOptions == 0) {
		if (arrowLocation == 1) {
			if (rfChannel > 1 && (joypad() & J_LEFT)) {
				rfChannel--;
				delay(50);
				refreshScreen = 1;
			}
			if (rfChannel < 125 && (joypad() & J_RIGHT)) {
				rfChannel++;
				delay(50);
				refreshScreen = 1;
			}
		}
		else if (arrowLocation == 2) {
			if (rfAddress > 1 && (joypad() & J_LEFT)) {
				rfAddress--;
				delay(50);
				refreshScreen = 1;
			}
			if (rfAddress < 254 && (joypad() & J_RIGHT)) {
				rfAddress++;
				delay(50);
				refreshScreen = 1;
			}
		}
		
		if (joypad() & J_DOWN) {
			arrowLocation = 2;
			refreshScreen = 1;
		}
		if (joypad() & J_UP) {
			arrowLocation = 1;
			refreshScreen = 1;
		}
	}
	
	// Transmit options
	if (transmitOptions == 0 && (joypad() & J_A)) {
		transmitOptions = 1;
		refreshScreen = 1;
		updatenrf(rfChannel << 8 | rfAddress);
	}
}

void checkInput() {
	// Blank out first character of last key press
	gotoxy(0,3);
	
	kpInput = 0;
	if (joypad() & J_A) {
		kpInput |= 0x01;
		//printf("A");
	}
	if (joypad() & J_B) {
		kpInput |= 0x02;
		//printf("B");
	}
	if (joypad() & J_RIGHT) {
		kpInput |= 0x04;
		//printf("R");
	}
	if (joypad() & J_LEFT) {
		kpInput |= 0x08;
		//printf("L");
	}
	if (joypad() & J_UP) {
		kpInput |= 0x10;
		//printf("U");
	}
	if (joypad() & J_DOWN) {
		kpInput |= 0x20;
		//printf("D");
	}
	if (joypad() & J_SELECT) {
		kpInput |= 0x40;
		//printf("T");
	}
	if (joypad() & J_START) {
		kpInput |= 0x80;
		//printf("S");
	}
	
	if (kpInput >= 1) {
		printf(".  ");
	}
	else if (kpInput == 0) {
		printf("   ");
	}
	
	senddata(kpInput);
}
